#include "dependency.h"

#include <regex>

#include "project.h"

const std::regex http("^https?://");

Dependency::Dependency(
    const std::string& path,
    Type type,
    const std::optional<std::string>& name,
    const std::optional<std::string>& version
):
    path(path),
    type(type),
    name(name),
    version(version)
{}

Dependency::Type Dependency::getType() const {
    return type;
}

std::optional<std::string> Dependency::getName() const {
    return name;
}

std::optional<std::string> Dependency::getVersion() const {
    return version;
}

bool Dependency::prepare(const std::filesystem::path& source, const std::filesystem::path& destination) {
    if (std::regex_search(path, http)) {
        Project::info("downloading project from " + path);
        std::filesystem::create_directories(destination / "downloads");

        return download(destination);
    } else {
        Project::info("checking project at path " + path);
        std::filesystem::directory_entry srcDir(source / path);
        if (!srcDir.exists()) {
            Project::error("Project at " + path + " doesn't exist");
            return false;
        }
    }

    return true;
}

bool Dependency::download(const std::filesystem::path& destination) {
    std::filesystem::path dwn = destination / "downloads";
    bool result = std::filesystem::create_directories(dwn);
    // if (!result) {
    //     Project::error("couldn't create " + dwn.string());
    //     return result;
    // }

    CURL *curl_handle;
    std::filesystem::path out = dwn / ("out" + std::to_string(std::rand()));
    FILE *pagefile;

    curl_global_init(CURL_GLOBAL_ALL);
    /* init the curl session */
    curl_handle = curl_easy_init();
    /* set URL to get here */
    curl_easy_setopt(curl_handle, CURLOPT_URL, path.c_str());
    /* disable progress meter, set to 0L to enable it */
    curl_easy_setopt(curl_handle, CURLOPT_NOPROGRESS, 1L);
    /* send all data to this function  */
    curl_easy_setopt(curl_handle, CURLOPT_WRITEFUNCTION, write);

    /* open the file */
    pagefile = fopen(out.c_str(), "wb");
    if (pagefile) {
        /* write the page body to this file handle */
        curl_easy_setopt(curl_handle, CURLOPT_WRITEDATA, pagefile);
        /* get it! */
        curl_easy_perform(curl_handle);
        /* close the header file */
        fclose(pagefile);
    } else {
        Project::error(std::string("couldn't open output file ") + out.c_str());
    }

    /* cleanup curl stuff */
    curl_easy_cleanup(curl_handle);
    curl_global_cleanup();

    return true;
}


size_t Dependency::write(void* file, size_t size, size_t nmemb, void* stream) {
    size_t written = fwrite(file, size, nmemb, (FILE *)stream);
    return written;
}
